.table_timers thead th {
    text-align: center
}
.table_timers tbody td {
    text-align: center;    
    word-wrap: break-word;
}
.table_timers tbody td:nth-child(1) { white-space: nowrap; }
.table_timers tbody td:nth-child(2) { white-space: nowrap; }

/*
.nav-tabs > li {
    font-weight: bold;
}
*/

.flex-container {
    display: flex;
    margin: 0 auto;    
}

.flex-container > div {
    align-self: center;
    padding-left: 3px;
}

.flex-container > div:first-child {
    align-self: center;
    padding-left: 20px;
}


#timerboardImgScreenshot {
    max-height: 450px;
}

.nowrap {
    white-space: nowrap
}